/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision$
 */ 

/** @file
*/

/** @defgroup hal_io hal_io
 * @{
 * @ingroup hal
 * @brief Macros input/output management
*/

#include "hal_platform.h"

#ifndef HAL_IO_H__
#define HAL_IO_H__

#define HAL_IO_P0_00 0, 0
#define HAL_IO_P0_01 0, 1
#define HAL_IO_P0_02 0, 2
#define HAL_IO_P0_03 0, 3
#define HAL_IO_P0_04 0, 4
#define HAL_IO_P0_05 0, 5
#define HAL_IO_P0_06 0, 6
#define HAL_IO_P0_07 0, 7
#define HAL_IO_P0_08 0, 8
#define HAL_IO_P0_09 0, 9

#define HAL_IO_P1_00 1, 0
#define HAL_IO_P1_01 1, 1
#define HAL_IO_P1_02 1, 2
#define HAL_IO_P1_03 1, 3
#define HAL_IO_P1_04 1, 4
#define HAL_IO_P1_05 1, 5
#define HAL_IO_P1_06 1, 6
#define HAL_IO_P1_07 1, 7

#define HAL_IO_P2_00 2, 0
#define HAL_IO_P2_01 2, 1
#define HAL_IO_P2_02 2, 2
#define HAL_IO_P2_03 2, 3
#define HAL_IO_P2_04 2, 4
#define HAL_IO_P2_05 2, 5
#define HAL_IO_P2_06 2, 6
#define HAL_IO_P2_07 2, 7
#define HAL_IO_P2_08 2, 8
#define HAL_IO_P2_09 2, 9
#define HAL_IO_P2_10 2, 10
#define HAL_IO_P2_11 2, 11

#define HAL_IO_INPUT_DIR  &= ~
#define HAL_IO_OUTPUT_DIR |=

#define HAL_IO_OUTPUT 0
#define HAL_IO_INPUT 1

#define HAL_IO_OUTPUT_NORMAL_STRENGTH 0x00
#define HAL_IO_OUTPUT_HIGH_STRENGTH 0x60
#define HAL_IO_INPUT_BUF_ON_NO_PULL 0x00
#define HAL_IO_INPUT_BUF_ON_PULL_DOWN 0x20
#define HAL_IO_INPUT_BUF_ON_PULL_UP 0x40
#define HAL_IO_INPUT_BUF_OFF 0x60

#define HAL_IO_CONFIG_INTERNAL(port_name, io_num, io_dir, is_input, io_mode)\
  do{\
    LPC_GPIO##port_name->DIR io_dir(1<<io_num);\
  }while(0)
/*port_name##CON = (io_mode)|(is_input<<4)|(io_num);\*/


#define HAL_IO_SET_STATE_INTERNAL(port_name, io_num, io_state)\
  do{\
    if (io_state == 1) {LPC_GPIO##port_name->DATA |= (1UL << io_num);} else { LPC_GPIO##port_name->DATA &= ~(1UL << io_num); }\
  }while(0)

#define HAL_IO_READ_INTERNAL(port_name, io_num) ((LPC_GPIO##port_name->DATA & (1UL >> io_num)) == (1UL >> io_num))



/**@brief Macro to configure an I/O.
 *  @details
 *  This macro configures a given I/O to input or output with pullup/buffer configuration.
 *  @param io_name I/O to configure.
 *  @param is_input Indicate if the I/O is to be set as an input.
 *  @param io_mode Pull resistor and buffer configuration (must be HAL_IO_OUTPUT_NORMAL_STRENGTH, HAL_IO_OUTPUT_HIGH_STRENGTH, HAL_IO_INPUT_BUF_ON_NO_PULL,
 *  HAL_IO_INPUT_BUF_ON_PULL_DOWN, HAL_IO_INPUT_BUF_ON_PULL_UP, or HAL_IO_INPUT_BUF_OFF).
*/
#define HAL_IO_CONFIG(io_name, is_input, io_mode) HAL_IO_CONFIG_INTERNAL(io_name, is_input##_DIR, is_input, io_mode)

/**@brief Macro to set an output.
 *  @details
 *  This macro sets the given output to the given level (1 -> high, 0 -> low).
 *  @param io_name Output to change.
 *  @param io_state Level to set.
*/
#define HAL_IO_SET_STATE(io_name, io_state)\
  HAL_IO_SET_STATE_INTERNAL(io_name, io_state)

/**@brief Macro that reads the current state of an input.
 *  @details
 *  This macro reads the current state of a logical input.
 *  @param io_name Input to read.
 *  @return Input state level (1 if high, 0 if low).
*/
#define HAL_IO_READ(io_name) HAL_IO_READ_INTERNAL(io_name)


#endif //HAL_IO_H__

/** @} */
