/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision$
 */

#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include <cr_section_macros.h>
#include <NXP/crp.h>

// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP ;

// Project specific includes
#include "hal_platform.h"
#include "lib_aci.h"
#include "lib_traces.h"
#include "system.h"
#include "services.h"
#include <string.h>



#ifdef SERVICES_PIPE_TYPE_MAPPING_CONTENT
    static KEYWORD_CODE services_pipe_type_mapping_t
        services_pipe_type_mapping[NUMBER_OF_PIPES] = SERVICES_PIPE_TYPE_MAPPING_CONTENT;
#else
    #define NUMBER_OF_PIPES 0
    static services_pipe_type_mapping_t * services_pipe_type_mapping = NULL;
#endif

static KEYWORD_CODE hal_aci_data_t
    setup_msgs[NB_SETUP_MESSAGES] = SETUP_MESSAGES_CONTENT;

#define ACI_CMD_Q_SIZE 16

static uint8_t data_credit_available;
static hal_aci_data_t aci_data;
static hal_aci_data_t aci_cmd;
static struct aci_state_t aci_state;


void main_application_loop();

int main(void)
{
  if (NULL != services_pipe_type_mapping)
  {
    aci_state.aci_setup_info.services_pipe_type_mapping = &services_pipe_type_mapping[0];
  }
  else
  {
    aci_state.aci_setup_info.services_pipe_type_mapping = NULL;
  }
  aci_state.aci_setup_info.number_of_pipes    = NUMBER_OF_PIPES;
  aci_state.aci_setup_info.setup_msgs         = setup_msgs;
  aci_state.aci_setup_info.num_setup_msgs     = NB_SETUP_MESSAGES;

  /** We reset the nRF8001 here by toggling the RESET line connected to the nRF8001
   *  and initialize the data structures required to setup the nRF8001
   */
  lib_aci_init(&aci_state);

	for (;;)
	{
	  main_application_loop();
	}
}

void main_application_loop(void)
{
  aci_evt_t * aci_evt;

  // We enter the if statement only when there is a ACI event available to be processed
  if (lib_aci_event_get(&aci_state, &aci_data))
  {
    aci_evt = (aci_evt_t *)&(aci_data.buffer[0]);

    //@todo change this so that the commands and events can be processed here in a switch case instead of callbacks
    switch(aci_evt->evt_opcode)
    {
        case ACI_EVT_DEVICE_STARTED:
        {
          data_credit_available = aci_evt->params.device_started.credit_available;
          switch(aci_evt->params.device_started.device_mode)
          {
            case ACI_DEVICE_SETUP:
            {
              aci_state.device_state = ACI_DEVICE_SETUP;
              if (ACI_STATUS_TRANSACTION_COMPLETE != do_aci_setup(&aci_state))
              {
                while (1) ;
              }
            }
            break;

          case ACI_DEVICE_STANDBY:
            lib_aci_connect(180/* in seconds */, 0x0100 /* advertising interval 100ms*/);
            break;
          }
        }
        break; //ACI Device Started Event
      case ACI_EVT_CMD_RSP:
        //If an ACI command response event comes with an error -> stop
        if (ACI_STATUS_SUCCESS != aci_evt->params.cmd_rsp.cmd_status)
        {
          //ACI ReadDynamicData and ACI WriteDynamicData will have status codes of
          //TRANSACTION_CONTINUE and TRANSACTION_COMPLETE
          //all other ACI commands will have status code of ACI_STATUS_SCUCCESS for a successful command
          while (1);
        }
        break;
      default:

        break;
    }
  }
  else
  {
    // No event in the ACI Event queue
    // Arduino can go to sleep now
  }
}
